#include "decoder.h"
#include "ncurses.h"
#include <locale.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 
#include <poll.h>
#include <errno.h>

const int blockchars[16]={
	' ',     //  0000
	0x2597, //▗ 0001
	0x2596, //▖ 0010
	0x2584, //▄ 0011
	0x259D, //▝ 0100
	0x2590, //▐ 0101
	0x259E, //▞ 0110
	0x259F, //▟ 0111
	0x2598, //▘ 1000
	0x259A, //▚ 1001
	0x258C, //▌ 1010
	0x2599, //▙ 1011
	0x2580, //▀ 1100
	0x259C, //▜ 1101
	0x259B, //▛ 1110
	0x2588};//█ 1111
int imgnum=0;

void print_image(BDG_IMAGE *img)
{
	if (img==NULL) return;
	move(0,0);
	int x;
	int y;
	for (y=0;y<img->height/2;y++)
	for (x=0;x<img->width;x++) {
		uint8_t u=img->img[x+(y*2+0)*img->width];
		uint8_t l=img->img[x+(y*2+1)*img->width];
		int n;
		move(y,x*4);
		for (n=0; n<4; n++) {
			int c;
			c=(u>>(6-(n*2))&0x03); //AB
			c=(l>>(6-(n*2))&0x03)|(c<<2); //CD
			//=> ABCD
			printw("%lc",blockchars[c]);
		}
	}
//	move(img->height/2,1);
//	printw("%d",imgnum++);
}


void test(BDG_IMAGE *img){
	memset(img->img,0,img->width*img->height);
	int x;
	int y;
	for (x=0;x<img->width*8;x++)
	for (y=0;y<img->height;y++) {
		int d=(x*x)+(y*y);
		int c=0;
		if (d<2000) c=1;
		int p=y*img->width+x/8;
		img->img[p]=img->img[p]|(c<<(7-x%8));
	}
}


void error(const char *msg)
{
    perror(msg);
    exit(0);
}

int sockfd=-1;

int simplepoll(int events, int time)
{
	
	struct pollfd ufd;
	ufd.fd=sockfd;
	ufd.events = events;
	return poll(&ufd,1,time);
}


/* getchar
 *  returns the character
*/
int ngetchar(int b)
{
//	printw("ngetchar %d ",b);
	uint8_t c;
	if (b==0) { //blocking read
		simplepoll(POLLIN,-1);
	} else simplepoll(POLLIN,50);
	int res=read(sockfd,&c,1);
	if (res==-1) {
		if (errno==EAGAIN) return -1;
		if (errno==EWOULDBLOCK) return -1;
		return -2;
	}
//	printw("%d %d XXX",c,res);
	if (res!=1) return -1;
	return c;
}

void nputchar(int c)
{
	write(sockfd,&c,1);
}

int main(int argc, char *argv[])
{
	if (argc!=3) {
		fprintf(stderr, "Usage: %s hostname port\n",argv[0]);
		exit(0);
	}
	int res=0;
	uint16_t portno=atoi(argv[2]);
	sockfd=socket(AF_INET, SOCK_STREAM | SOCK_NONBLOCK, 0);
	struct sockaddr_in serv_addr;
	struct hostent *server;
	server=gethostbyname(argv[1]);
	if (server==NULL) {
		fprintf(stderr, "Error, host %s not found\n",argv[1]);
		exit(0);
	}
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
	serv_addr.sin_port = htons(portno);
	fprintf(stderr, "Connecting...");
	res=connect(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr));
	//if (res!=EINPROGRESS) error("error connecting"); else
	if (res<0) {
		res=simplepoll(POLLOUT,-1);
		if (res<0) error("Irgendwas");
	}
	fprintf(stderr, "done\n");
	setlocale(LC_ALL, "");
	BDG_IMAGE img;
	img.gc=ngetchar;
	img.pc=nputchar;
	initscr();			/* Start curses mode 		  */
	set_resolution(&img,COLS/8*2,LINES*2);
	clear();
	noecho();
	cbreak();
	halfdelay(5); 
	keypad(stdscr, TRUE);
	res=1;
	while (true) {
		res=decode(&img);
		if (res==1) print_image(&img);
		refresh();
		if (res==0) {
			int ch=getch();
			if (ch==KEY_LEFT)  nputchar('h'); else
			if (ch==KEY_RIGHT) nputchar('l'); else
			if (ch==KEY_UP)    nputchar('k'); else
			if (ch==KEY_DOWN)  nputchar('j'); else
			if (ch==' ') nputchar('\n');
		}
	//	sleep(1);
		//getch();
	}
	endwin();			/* End curses mode		  */
	close(sockfd);
	return 0;
}
